/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.ha.framework.server;

import java.net.InetAddress;
import org.jboss.ha.framework.interfaces.ClusterNode;
import org.jgroups.Address;

/**
 * Replacement for a JG IpAddress that doesn't base its representation
 * on the JG address but on the computed node name added to the IPAddress instead.
 * This is to avoid any problem in the cluster as some nodes may interpret a node name
 * differently (IP resolution, name case, FQDN or host name, etc.)
 *
 * @see org.jboss.ha.framework.server.ClusterPartitionMBean
 *
 * @author  <a href="mailto:sacha.labourey@jboss.org">Sacha Labourey</a>.
 * @author Brian Stansberry
 * @author Vladimir Blagojevic
 * @author <a href="mailto:galder.zamarreno@jboss.com">Galder Zamarreno</a>
 * @version $Revision: 104456 $
 */

public class ClusterNodeImpl
   implements ClusterNode
{
   // Constants -----------------------------------------------------
   
   /** The serialVersionUID */
   private static final long serialVersionUID = -1831036833785680731L;
   
   // Attributes ----------------------------------------------------
  
   private final String id;
   private final Address jgAddress;
   private final AddressPort address;

   // Static --------------------------------------------------------
   
   // Constructors --------------------------------------------------
       
   ClusterNodeImpl(String id, Address jgAddress, AddressPort addressPort)
   {
      if (id == null)
      {
         throw new IllegalArgumentException("Null id");
      }
      if (addressPort == null)
      {
         throw new IllegalArgumentException("Null addressPort");
      }
      this.id = id;
      this.address = addressPort;
      
      this.jgAddress = jgAddress;
      
   }
  
   // Public --------------------------------------------------------

   public String getName()
   {
      return this.id;
   }
   
   public InetAddress getIpAddress()
   {
      return this.address.getInetAddress();
   }
   
   public int getPort()
   {
      return this.address.getPort();      
   }

   // Package protected ----------------------------------------------

   Address getOriginalJGAddress()
   {
      return this.jgAddress;
   }
   
   // Comparable implementation ----------------------------------------------

   public int compareTo(ClusterNode o)
   {
      if (o == null)
         throw new ClassCastException("Comparison to null value");
      
      if (!(o instanceof ClusterNodeImpl))
         throw new ClassCastException("Comparison between different classes");

      return this.id.compareTo(o.getName());
   }
   
   // java.lang.Object overrides ---------------------------------------------------

   public boolean equals(Object obj)
   {
      if (obj == null || !(obj instanceof ClusterNodeImpl)) return false;
      
      ClusterNodeImpl other = (ClusterNodeImpl) obj;
      return this.id.equals(other.id);
   }

   public int hashCode()
   {
      return id.hashCode();
   }

   public String toString()
   {
      return this.getName();
   }

   // Package protected ---------------------------------------------
   
   // Protected -----------------------------------------------------


   // Private -------------------------------------------------------
   
   /**
    * Tries to determine the hostname of the given InetAddress without 
    * triggering a reverse DNS lookup.  Tries to parse a symbolic hostname 
    * from {@link InetAddress.toString()}, which is documented to return a 
    * String of the form "symbolicname/ipaddress" with 'symbolicname' blank 
    * if not stored in the object.
    * <p/>
    * If the symbolic name cannot be determined from InetAddress.toString(),
    * the value of {@link InetAddress.getHostAddress()} is returned.
    */
   private static String getFastHostName(InetAddress address)
   {
      String result = null;
      
      String hostAddress = address.getHostAddress();
      
      String inetAddr = address.toString();
      int idx = inetAddr.lastIndexOf('/');
      int idx1 = inetAddr.indexOf(hostAddress);
      if (idx1 == idx + 1)
      {
         if (idx == 0)
            result = hostAddress;
         else
            result = inetAddr.substring(0, idx);
      }
      else
      {
         // Doesn't follow the toString() contract!
         result = hostAddress;
      }
      return result;
   }
   
   // Inner classes -------------------------------------------------
   
}
